//
// C++ Interface: condition
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _SUCS_CONDITION_H
#define _SUCS_CONDITION_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#ifdef __SUCS
#include "mutex.h"
#else
#include <sucs/mutex.h>
#endif
using namespace sucs;

namespace sucs
{

/**
@author Gav Wood
*/

class Condition
{
	mutable pthread_cond_t *theVar;

protected:
	const Mutex *theMutex;

public:
	/** Unblocks >= 1 waiter.
	 * If useMutex is true or mutex is passed, then the mutex is forst locked.
	 */
	void signal(bool useMutex = true) const;
	void signal(const Mutex &mutex) const;
	
	/** Unblocks all waiters.
	 * If useMutex is true or mutex is passed, then the mutex is forst locked.
	 */
	void broadcast(bool useMutex = true) const;
	void broadcast(const Mutex &mutex) const;
	
	/** Sets Mutex to be used with this Condition lock. */
	void setMutex(const Mutex &mutex);
	
	/** Waits; mutex *must* be preset with the constructor or @setMutex, and must be locked. */
	void wait() const;
	
	/** Waits; mutex for locking is passed; must already be locked. */
	void wait(const Mutex &mutex) const;

	/** Waits; mutex *must* be preset with the constructor or @setMutex, is locked before waiting. */
	void lockAndWait() const;
	
	/** Waits; mutex for locking is passed; is locked before waiting. */
	void lockAndWait(const Mutex &mutex) const;

//	bool wait(const Time &time); //TODO: time doesn't work yet

	Condition();
	Condition(const Mutex &mutex);
	~Condition();
};

};

#endif
