//
// C++ Implementation: mutex
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define __SUCS 1

#include <iostream>
#include <cassert>
using namespace std;

#include <errno.h>
#include <pthread.h>

#include "mutex.h"
using namespace sucs;

#define mutexDebug 0

namespace sucs
{

Mutex::Mutex(string name)
{
	theName = name;
	if(mutexDebug) cerr << "Mutex " << theName << " (" << this << "): " << pthread_self() << ": Creating Mutex..." << endl << flush;
	theVar = new pthread_mutex_t;
	assert(!pthread_mutex_init(theVar, NULL));
}

Mutex::~Mutex()
{
	if(mutexDebug) cerr << "Mutex " << theName << " (" << this << "): " << pthread_self() << ": Destoying Mutex..." << endl << flush;
	int ret = pthread_mutex_destroy(theVar);
	if(ret)
		cerr << "ERROR: " << strerror(errno) << endl << flush;
	assert(!ret);
	delete theVar;
}

bool Mutex::lock() const
{
	if(mutexDebug) cerr << "Mutex " << theName << " (" << this << "): " << pthread_self() << ": Locking Mutex..." << flush;
	int ret = pthread_mutex_lock(theVar);
	assert(ret != EINVAL && ret != EAGAIN);
	if(mutexDebug) cerr << "Locked." << endl << flush;
	return !ret;
}

bool Mutex::tryLock() const
{
	int ret = pthread_mutex_trylock(theVar);
	assert(ret != EINVAL && ret != EAGAIN);
	return !ret;
}

bool Mutex::unlock() const
{
	if(mutexDebug) cerr << "Mutex " << theName << " (" << this << "): " << pthread_self() << ": Unlocking Mutex..." << flush;
	int ret = pthread_mutex_unlock(theVar);
	assert(ret != EINVAL && ret != EAGAIN);
	if(mutexDebug) cerr << "Unlocked." << endl << flush;
	return !ret;
}

};
