//
// C++ Interface: referencecounted
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef _SUCS_REFERENCECOUNTED_H
#define _SUCS_REFERENCECOUNTED_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <map>
using namespace std;

#include "sema.h"
using namespace sucs;

namespace sucs
{

/**
@author Gav Wood
*/

//TODO: Maybe inline a lot of this class, as it's so small?

class ReferenceCounted
{
	static unsigned int nextRef;
	static map<unsigned int, unsigned int> theRefs;
	static Sema theLock;
	unsigned int refId;

protected:
	/**
	 * Use this to check whether the object should really be destroyed in all derived classes'
	 * destructors.
	 */
	const bool lastInstance();
	
	/**
	 * Use this in your derived classes' operator= overload.
	 */
	ReferenceCounted &operator=(const ReferenceCounted &source);

	/**
	 * Default constructor; creates a new reference.
	 */
	ReferenceCounted();
	
	/**
	 * Make sure this gets called in any derived classes' copy-constructors.
	 */
	ReferenceCounted(const ReferenceCounted &copy);
	
	/**
	 * Simple destructor.
	 */
	virtual ~ReferenceCounted();

public:
	/**
	 * Get the reference line of this object. Useful for keeping track of objects.
	 */
	const unsigned int reference() const;
};

};

#endif
