/***************************************************************************
                          link.h  -
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef _SUCS_LINK_H
#define _SUCS_LINK_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <string>
#include <map>
#include <iostream>
using namespace std;

#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/types.h>
#include <sys/socket.h>

#ifdef __SUCS
#include "conditionmutex.h"
#include "sema.h"
#include "socket.h"
#else
#include <sucs/conditionmutex.h>
#include <sucs/sema.h>
#include <sucs/socket.h>
#endif
using namespace sucs;

namespace sucs
{

class Mast;

class Link: public Socket, private Sema
{
	Mast *theMast;
	
	void setMast(Mast *mast) { theMast = mast; }
	friend class Mast;
	
	virtual void cleanup();

protected:
	/** Returns a pointer to the Link's associated mast. NULL if no Mast associated. */
	Mast *mast() { return theMast; }
	
	/** Returns a pointer to the SIGIO ConditionMutex. NULL if no Mast associated. */
	const ConditionMutex *gotIO();
	
	/** Override to send first stuff on a socket connect(). */
	virtual void init() {}
	
	/** Override to send final stuff on a socket close(). */
	virtual void deinit() {}

public:
	const string name() const;

	void adopt(int socket);
	const bool connect(const string anAddress, int Port);
	void disconnect();

	const bool send(const string &theString) const;
	const bool send(const char *Buffer, int Len) const;
	const bool send(const unsigned char Data) const;

	/**
	 * Waits for input in such a way to allow the thread to cancel in the meantime.
	 * SHOULD ONLY BE CALLED WHEN A MAST IS ASSOCIATED WITH THIS LINK!
	 *
	 * If used, cleanupWait should be called on thread cancellation.
	 */
	void waitForInput() const;
	void cleanupWait() { gotIO()->unlock(); }
	
	/**
	 * Receives next communication into buffer, upto a maximum of Len charecters.
	 */
	const int receive(char *Buffer, int Len, int timeOut = 0, int *timeLeft = 0);
	
	/**
	 * Receives Len charecters into Buffer, blocking forever if neccessary
	 */
	void receiveChunk(char *Buffer, int Len, int timeOut = 0);
	
	/**
	 * Receives a line into string and returns it, blocking forever if neccessary.
	 */ 
	const string receiveLine();
	
	/**
	 * Receive charecters until ONE OF THE CHARECTERS in until[] is found
	 * (will receive until whitespace is encountered by default)
	 */
	const string receiveUntil(char until[3] = "\n \t");
	
	/**
	 * Following methods neccessary for reference counting mechanism
	 */
	Link &operator=(const Link &src) { Socket::operator=(src); theMast = src.theMast; }
	
	Link();
	Link(int aSocket);
	Link(const Link &src);
	virtual ~Link() {}
};

}

#endif
