/***************************************************************************
                          host.h  -  Mast network definition file
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __MAST
#define __MAST

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <iostream>
#include <vector>
#include <list>
using namespace std;

#ifdef __SUCS
#include "thread.h"
#include "sema.h"
#include "link.h"
#include "cleaner.h"
#include "signalhandler.h"
#include "socket.h"
#include "conditionmutex.h"
#else
#include <sucs/thread.h>
#include <sucs/sema.h>
#include <sucs/link.h>
#include <sucs/cleaner.h>
#include <sucs/signalhandler.h>
#include <sucs/socket.h>
#include <sucs/conditionmutex.h>
#endif
using namespace sucs;

namespace sucs
{

class Mast: public Socket, protected SignalHandler, protected Thread
{
	friend class Link;
	
	int thePort;
	bool theListening;
	static Cleaner<Link> theCleaner;
	Sema theQuitHandler, theQuitWaiter, theLinksAccess;
	ConditionMutex theGotIO;
	
	int theSocket;	// should be fd

	virtual void handler(int);
	
protected: // for SoapMast only
	virtual const bool bind(int newPort);
	virtual const bool listen();
	virtual void tryAccept();
	
	virtual void body();
	virtual void ghost();
	
protected:
	list<Link *> theLinks;

	virtual Link *newArriver() { return new Link; }

public:
	/** Opens the doors to incoming connections. */
	const bool start(int newPort = -1);
	
	/**
	 * Usually called from the owner fo the Mast.
	 * Used to block the process until the mast is done; then cleans up any links.
	 */
	void handleQuit();
	
	/**
	 * Used to block the process until the mast is done.
	 */
	void waitForQuit();
	
	/**
	 * Usually called from a Link.
	 * Closes & deletes all links, exits listening thread and unblocks waitForQuit().
	 */
	void quit();

	/** Closes all Links from this Mast */
	void closeAll();
	
	/**
	 * Uses another thread to safely delete the Link given or all links.
	 * Does nothing if link was not initiated from this Mast.
	 */
	void safeDelete(Link *c);
	void deleteAll();

	Mast(int newPort = -1);
	~Mast();
};

}

#endif
