//
// C++ Interface: socket
//
// Description:
//
//
// Author: Gav Wood <gav@indigoarchive.net>, (C) 2004
//
// Copyright: See COPYING file that comes with this distribution
//
//
#ifndef SUCSSOCKET_H
#define SUCSSOCKET_H

#include <map>
using namespace std;

namespace sucs
{

/**
@author Gav Wood
*/

class Socket
{
	static map<int, int> refCount;

protected:
	int theSocket;

	/** Shuts down socket; needed for reference counting mechanism. Should be reimplemented
	 * if "shutting down" requires more than a close().
	 */ 
	virtual void cleanup();
	
public:
	/** Returns the open status of the socket. */
	inline const bool isOpen() const { return theSocket > -1; }
	
	/** Sets blocking I/O on a socket. */
	void setBlocking(bool blocking) const;
	
	/**
	 * Checks whether there is some input waiting to be received.
	 */
	const bool poll() const;
	
	/** Closes socket. */
	void close();

	/** Can be used to duplicate a socket.
	 * Reference counted, so that the socket is closed when the last Socket is destroyed.
	 */
	Socket &operator=(const Socket &src);

	/** Constructors/destructors. Reference counting mechanisms in place to make sure that
	 * closing of socket only takes place on the destruction of the last Socket of that
	 * socket.
	 */
	Socket();
	Socket(int aSocket);
	Socket(const Socket &src);
	virtual ~Socket();
};

};

#endif
