/***************************************************************************
                          factory.h
    Factory class for loading and utilising plugins.
                             -------------------
    begin                : Wed Jul 17 2002
    copyright            : (C) 2002 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __FACTORY_H
#define __FACTORY_H

#pragma interface

#include <iostream>
#include <string>

using namespace std;

#include <dlfcn.h>

namespace sucs
{

template<class Base> class Factory
{
	string theLastErrorVal;
	string theLibPath;

	void *theLibrary;
	void *theCreator;
	void *theIdentifier;
	void *theVersion;

public:
	inline string theLastError() { return theLastErrorVal; }
	inline void theLastError(string newVal) { theLastErrorVal = newVal; }

	// produces one instance of the opened plugin class. returns 0 if bad
	inline Base *createInstance() { if(isOpen()) return ((Base *(*)())(theCreator))(); return 0; }

	// opens the factory, for producing plugins of pluginName
	inline bool open(const string libPath);

	// returns false if failed to create factory
	inline bool isOpen() { return theLibrary && theCreator && theIdentifier; }

	// returns the plugin's class name
	inline string getID() { if(isOpen()) return ((char *(*)())(theIdentifier))(); return ""; }

	// returns the plugin's class name
	inline int getVersion() { if(isOpen()) return ((int(*)())(theVersion))(); return -1; }
	
	// closes the factory
	inline void close() { if(theLibrary) dlclose(theLibrary); theLibrary = 0; theCreator = 0; }

	// opens a factory ready for production
	inline Factory(const string libPath) { theLibrary = 0; theCreator = 0; open(libPath); }

	// creates a closed factory
	inline Factory() { theLibrary = 0; theCreator = 0; theLibPath = ""; }

	// safe copy constructor. bloody inefficient!
	inline Factory &operator=(const Factory &source)
	{
		if(source.theLibPath != "")
			open(source.theLibPath);
		else
			close();
		return *this;
	}

	inline ~Factory() { close (); }
};

template<class Base> bool Factory<Base>::open(const string libPath)
{
	close();

	theLibPath = libPath;

	theLibrary = dlopen(libPath.c_str(), RTLD_GLOBAL | RTLD_LAZY);
	char *err = dlerror();
	if(err)
	{
		close();
		theLastError(err);
		cerr << "Failed to load plugin " << libPath << ": " << err << endl;
		return false;
	}

	theCreator = dlsym(theLibrary, "createInstance");
	err = dlerror();
	if(err)
	{
		close();
		theLastError(err);
		cerr << "Failed to load plugin " << libPath << ": " << err << endl;
		return false;
	}

	theIdentifier = dlsym(theLibrary, "getID");
	err = dlerror();
	if(err)
	{
		close();
		theLastError(err);
		cerr << "Failed to load plugin " << libPath << ": " << err << endl;
		return false;
	}

	theVersion = dlsym(theLibrary, "getVersion");
	err = dlerror();
	if(err)
	{
		close();
		theLastError(err);
		cerr << "Failed to load plugin " << libPath << ": " << err << endl;
		return false;
	}
	
	return true;
}

}

#endif
