/***************************************************************************
                          buffer.h  -  Main Buffer header file
                             -------------------
    begin                : Tue 20 November 2001
    copyright            : (C) 2001 by Gav Wood
    email                : gav@indigoarchive.net
 ***************************************************************************/

#ifndef __BUFFER
#define __BUFFER

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#pragma interface

#include <vector>
using namespace std;

namespace sucs
{

class Buffer
{
	float theFrequency;
	vector<float> theSamples;

public:
	float &operator[](unsigned index) { return theSamples[index]; }
	const float &operator[](unsigned index) const { return theSamples[index]; }

	Buffer subBuffer(unsigned index, unsigned length = 0) const;

	unsigned size() const { return theSamples.size(); }

	void resizeForFrequency(float freq);
	void setFrequency(float freq) { theFrequency = freq; }
	float frequency() const { return theFrequency; }

	void applyHanning();

	// creation commands
	void addPureTone(float freq, float mag = 1.0);
	void clear();

	void copyFromShort(unsigned numSamples, signed short samples[] = NULL, float freq = 0);
	void copyFromLong(unsigned numSamples, signed long samples[] = NULL, float freq = 0);

	Buffer(float freq = 0, float secs = 0);
	Buffer(unsigned numSamples, float samples[] = NULL, float freq = 0);		// NOTE: Major API change
	Buffer(const Buffer &source) { theFrequency = source.theFrequency; theSamples = source.theSamples; }
	~Buffer();
};

}

#endif

